/*******************************************************************************
* File Name: InputCounter_PM.c  
* Version 2.20
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "InputCounter.h"

static InputCounter_backupStruct InputCounter_backup;


/*******************************************************************************
* Function Name: InputCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  InputCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void InputCounter_SaveConfig(void) 
{
    #if (!InputCounter_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            InputCounter_backup.CounterUdb = InputCounter_ReadCounter();
            InputCounter_backup.CounterPeriod = InputCounter_ReadPeriod();
            InputCounter_backup.CompareValue = InputCounter_ReadCompare();
            InputCounter_backup.InterruptMaskValue = InputCounter_STATUS_MASK;
        #endif /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            InputCounter_backup.CounterUdb = InputCounter_ReadCounter();
            InputCounter_backup.InterruptMaskValue = InputCounter_STATUS_MASK;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!InputCounter_ControlRegRemoved)
            InputCounter_backup.CounterControlRegister = InputCounter_ReadControlRegister();
        #endif /* (!InputCounter_ControlRegRemoved) */
    #endif /* (!InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  InputCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void InputCounter_RestoreConfig(void) 
{      
    #if (!InputCounter_UsingFixedFunction)     
        /* Restore the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            /* Interrupt State Backup for Critical Region*/
            uint8 InputCounter_interruptState;
        
            InputCounter_WriteCounter(InputCounter_backup.CounterUdb);
            InputCounter_WritePeriod(InputCounter_backup.CounterPeriod);
            InputCounter_WriteCompare(InputCounter_backup.CompareValue);
            /* Enter Critical Region*/
            InputCounter_interruptState = CyEnterCriticalSection();
        
            InputCounter_STATUS_AUX_CTRL |= InputCounter_STATUS_ACTL_INT_EN_MASK;
            /* Exit Critical Region*/
            CyExitCriticalSection(InputCounter_interruptState);
            InputCounter_STATUS_MASK = InputCounter_backup.InterruptMaskValue;
        #endif  /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            InputCounter_WriteCounter(InputCounter_backup.CounterUdb);
            InputCounter_STATUS_MASK = InputCounter_backup.InterruptMaskValue;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!InputCounter_ControlRegRemoved)
            InputCounter_WriteControlRegister(InputCounter_backup.CounterControlRegister);
        #endif /* (!InputCounter_ControlRegRemoved) */
    #endif /* (!InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  InputCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void InputCounter_Sleep(void) 
{
    #if(!InputCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(InputCounter_CTRL_ENABLE == (InputCounter_CONTROL & InputCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            InputCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            InputCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!InputCounter_ControlRegRemoved) */
    InputCounter_Stop();
    InputCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: InputCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  InputCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void InputCounter_Wakeup(void) 
{
    InputCounter_RestoreConfig();
    #if(!InputCounter_ControlRegRemoved)
        if(InputCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            InputCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!InputCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
