/*******************************************************************************
* File Name: RefCounter_PM.c  
* Version 2.20
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "RefCounter.h"

static RefCounter_backupStruct RefCounter_backup;


/*******************************************************************************
* Function Name: RefCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  RefCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void RefCounter_SaveConfig(void) 
{
    #if (!RefCounter_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            RefCounter_backup.CounterUdb = RefCounter_ReadCounter();
            RefCounter_backup.CounterPeriod = RefCounter_ReadPeriod();
            RefCounter_backup.CompareValue = RefCounter_ReadCompare();
            RefCounter_backup.InterruptMaskValue = RefCounter_STATUS_MASK;
        #endif /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            RefCounter_backup.CounterUdb = RefCounter_ReadCounter();
            RefCounter_backup.InterruptMaskValue = RefCounter_STATUS_MASK;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!RefCounter_ControlRegRemoved)
            RefCounter_backup.CounterControlRegister = RefCounter_ReadControlRegister();
        #endif /* (!RefCounter_ControlRegRemoved) */
    #endif /* (!RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  RefCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void RefCounter_RestoreConfig(void) 
{      
    #if (!RefCounter_UsingFixedFunction)     
        /* Restore the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            /* Interrupt State Backup for Critical Region*/
            uint8 RefCounter_interruptState;
        
            RefCounter_WriteCounter(RefCounter_backup.CounterUdb);
            RefCounter_WritePeriod(RefCounter_backup.CounterPeriod);
            RefCounter_WriteCompare(RefCounter_backup.CompareValue);
            /* Enter Critical Region*/
            RefCounter_interruptState = CyEnterCriticalSection();
        
            RefCounter_STATUS_AUX_CTRL |= RefCounter_STATUS_ACTL_INT_EN_MASK;
            /* Exit Critical Region*/
            CyExitCriticalSection(RefCounter_interruptState);
            RefCounter_STATUS_MASK = RefCounter_backup.InterruptMaskValue;
        #endif  /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            RefCounter_WriteCounter(RefCounter_backup.CounterUdb);
            RefCounter_STATUS_MASK = RefCounter_backup.InterruptMaskValue;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!RefCounter_ControlRegRemoved)
            RefCounter_WriteControlRegister(RefCounter_backup.CounterControlRegister);
        #endif /* (!RefCounter_ControlRegRemoved) */
    #endif /* (!RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  RefCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void RefCounter_Sleep(void) 
{
    #if(!RefCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(RefCounter_CTRL_ENABLE == (RefCounter_CONTROL & RefCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            RefCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            RefCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!RefCounter_ControlRegRemoved) */
    RefCounter_Stop();
    RefCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: RefCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  RefCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void RefCounter_Wakeup(void) 
{
    RefCounter_RestoreConfig();
    #if(!RefCounter_ControlRegRemoved)
        if(RefCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            RefCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!RefCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
