/*******************************************************************************
* File Name: TimerOverflow.h
* Version 2.30
*
*  Description:
*     Contains the function prototypes and constants available to the timer
*     user module.
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#if !defined(CY_Timer_v2_30_TimerOverflow_H)
#define CY_Timer_v2_30_TimerOverflow_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component Timer_v2_30 requires cy_boot v3.0 or later
#endif /* (CY_ PSOC5LP) */


/**************************************
*           Parameter Defaults
**************************************/

#define TimerOverflow_Resolution                 32u
#define TimerOverflow_UsingFixedFunction         0u
#define TimerOverflow_UsingHWCaptureCounter      0u
#define TimerOverflow_SoftwareCaptureMode        0u
#define TimerOverflow_SoftwareTriggerMode        0u
#define TimerOverflow_UsingHWEnable              0u
#define TimerOverflow_EnableTriggerMode          0u
#define TimerOverflow_InterruptOnCaptureCount    0u
#define TimerOverflow_RunModeUsed                1u
#define TimerOverflow_ControlRegRemoved          0u


/***************************************
*       Type defines
***************************************/


/**************************************************************************
 * Sleep Wakeup Backup structure for Timer Component
 *************************************************************************/
typedef struct TimerOverflow_backupStruct
{
    uint8 TimerEnableState;
    #if(!TimerOverflow_UsingFixedFunction)
        #if (CY_PSOC5A)
            uint32 TimerUdb;                 /* Timer internal counter value */
            uint32 TimerPeriod;              /* Timer Period value       */
            uint8 InterruptMaskValue;       /* Timer Compare Value */
            #if (TimerOverflow_UsingHWCaptureCounter)
                uint32 TimerCaptureCounter;  /* Timer Capture Counter Value */
            #endif /* variable declaration for backing up Capture Counter value*/
        #endif /* variables for non retention registers in PSoC5A */

        #if (CY_PSOC3 || CY_PSOC5LP)
            uint32 TimerUdb;
            uint8 InterruptMaskValue;
            #if (TimerOverflow_UsingHWCaptureCounter)
                uint32 TimerCaptureCounter;
            #endif /* variable declarations for backing up non retention registers in PSoC3 or PSoC5LP */
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */

        #if (!TimerOverflow_ControlRegRemoved)
        uint8 TimerControlRegister;
        #endif /* variable declaration for backing up enable state of the Timer */
    #endif /* define backup variables only for UDB implementation. Fixed function registers are all retention */
}TimerOverflow_backupStruct;


/***************************************
*       Function Prototypes
***************************************/

void    TimerOverflow_Start(void) ;
void    TimerOverflow_Stop(void) ;

void    TimerOverflow_SetInterruptMode(uint8 interruptMode) ;
uint8   TimerOverflow_ReadStatusRegister(void) ;
/* Deprecated function. Do not use this in future. Retained for backward compatibility */
#define TimerOverflow_GetInterruptSource() TimerOverflow_ReadStatusRegister()

#if(!TimerOverflow_ControlRegRemoved)
    uint8   TimerOverflow_ReadControlRegister(void) ;
    void    TimerOverflow_WriteControlRegister(uint8 control) \
        ;
#endif /* (!TimerOverflow_ControlRegRemoved) */

uint32  TimerOverflow_ReadPeriod(void) ;
void    TimerOverflow_WritePeriod(uint32 period) \
    ;
uint32  TimerOverflow_ReadCounter(void) ;
void    TimerOverflow_WriteCounter(uint32 counter) \
    ;
uint32  TimerOverflow_ReadCapture(void) ;
void    TimerOverflow_SoftwareCapture(void) ;


#if(!TimerOverflow_UsingFixedFunction) /* UDB Prototypes */
    #if (TimerOverflow_SoftwareCaptureMode)
        void    TimerOverflow_SetCaptureMode(uint8 captureMode) ;
    #endif /* (!TimerOverflow_UsingFixedFunction) */

    #if (TimerOverflow_SoftwareTriggerMode)
        void    TimerOverflow_SetTriggerMode(uint8 triggerMode) ;
    #endif /* (TimerOverflow_SoftwareTriggerMode) */
    #if (TimerOverflow_EnableTriggerMode)
        void    TimerOverflow_EnableTrigger(void) ;
        void    TimerOverflow_DisableTrigger(void) ;
    #endif /* (TimerOverflow_EnableTriggerMode) */

    #if(TimerOverflow_InterruptOnCaptureCount)
        #if(!TimerOverflow_ControlRegRemoved)
            void    TimerOverflow_SetInterruptCount(uint8 interruptCount) \
                ;
        #endif /* (!TimerOverflow_ControlRegRemoved) */
    #endif /* (TimerOverflow_InterruptOnCaptureCount) */

    #if (TimerOverflow_UsingHWCaptureCounter)
        void    TimerOverflow_SetCaptureCount(uint8 captureCount) \
            ;
        uint8   TimerOverflow_ReadCaptureCount(void) ;
    #endif /* (TimerOverflow_UsingHWCaptureCounter) */

    void TimerOverflow_ClearFIFO(void) ;
#endif /* UDB Prototypes */

/* Sleep Retention APIs */
void TimerOverflow_Init(void)          ;
void TimerOverflow_Enable(void)        ;
void TimerOverflow_SaveConfig(void)    ;
void TimerOverflow_RestoreConfig(void) ;
void TimerOverflow_Sleep(void)         ;
void TimerOverflow_Wakeup(void)        ;


/***************************************
*   Enumerated Types and Parameters
***************************************/

/* Enumerated Type B_Timer__CaptureModes, Used in Capture Mode */
#define TimerOverflow__B_TIMER__CM_NONE 0
#define TimerOverflow__B_TIMER__CM_RISINGEDGE 1
#define TimerOverflow__B_TIMER__CM_FALLINGEDGE 2
#define TimerOverflow__B_TIMER__CM_EITHEREDGE 3
#define TimerOverflow__B_TIMER__CM_SOFTWARE 4



/* Enumerated Type B_Timer__TriggerModes, Used in Trigger Mode */
#define TimerOverflow__B_TIMER__TM_NONE 0x00u
#define TimerOverflow__B_TIMER__TM_RISINGEDGE 0x04u
#define TimerOverflow__B_TIMER__TM_FALLINGEDGE 0x08u
#define TimerOverflow__B_TIMER__TM_EITHEREDGE 0x0Cu
#define TimerOverflow__B_TIMER__TM_SOFTWARE 0x10u


/***************************************
*    Initialial Parameter Constants
***************************************/

#define TimerOverflow_INIT_PERIOD             59999999u
#define TimerOverflow_INIT_CAPTURE_MODE       (0u << TimerOverflow_CTRL_CAP_MODE_SHIFT)
#define TimerOverflow_INIT_TRIGGER_MODE       (0u << TimerOverflow_CTRL_TRIG_MODE_SHIFT)
#if (TimerOverflow_UsingFixedFunction)
    #define TimerOverflow_INIT_INTERRUPT_MODE ((1u << TimerOverflow_STATUS_TC_INT_MASK_SHIFT) | \
                                                  (0 << TimerOverflow_STATUS_CAPTURE_INT_MASK_SHIFT))
#else
    #define TimerOverflow_INIT_INTERRUPT_MODE ((1u << TimerOverflow_STATUS_TC_INT_MASK_SHIFT) | \
                                                 (0 << TimerOverflow_STATUS_CAPTURE_INT_MASK_SHIFT) | \
                                                 (0 << TimerOverflow_STATUS_FIFOFULL_INT_MASK_SHIFT))
#endif /* (TimerOverflow_UsingFixedFunction) */
#define TimerOverflow_INIT_CAPTURE_COUNT      (2u)
#define TimerOverflow_INIT_INT_CAPTURE_COUNT  ((1u - 1) << TimerOverflow_CTRL_INTCNT_SHIFT)


/***************************************
*           Registers
***************************************/

#if (TimerOverflow_UsingFixedFunction) /* Implementation Specific Registers and Register Constants */


    /***************************************
    *    Fixed Function Registers
    ***************************************/

    #define TimerOverflow_STATUS         (*(reg8 *) TimerOverflow_TimerHW__SR0 )
    /* In Fixed Function Block Status and Mask are the same register */
    #define TimerOverflow_STATUS_MASK    (*(reg8 *) TimerOverflow_TimerHW__SR0 )
    #define TimerOverflow_CONTROL        (*(reg8 *) TimerOverflow_TimerHW__CFG0)
    #define TimerOverflow_CONTROL2       (*(reg8 *) TimerOverflow_TimerHW__CFG1)
    #define TimerOverflow_CONTROL2_PTR   ( (reg8 *) TimerOverflow_TimerHW__CFG1)
    #define TimerOverflow_RT1            (*(reg8 *) TimerOverflow_TimerHW__RT1)
    #define TimerOverflow_RT1_PTR        ( (reg8 *) TimerOverflow_TimerHW__RT1)

    #if (CY_PSOC3 || CY_PSOC5LP)
        #define TimerOverflow_CONTROL3       (*(reg8 *) TimerOverflow_TimerHW__CFG2)
        #define TimerOverflow_CONTROL3_PTR   ( (reg8 *) TimerOverflow_TimerHW__CFG2)
    #endif /* (CY_PSOC3 || CY_PSOC5LP) */
    #define TimerOverflow_GLOBAL_ENABLE  (*(reg8 *) TimerOverflow_TimerHW__PM_ACT_CFG)
    #define TimerOverflow_GLOBAL_STBY_ENABLE  (*(reg8 *) TimerOverflow_TimerHW__PM_STBY_CFG)

    #define TimerOverflow_CAPTURE_LSB         (* (reg16 *) TimerOverflow_TimerHW__CAP0 )
    #define TimerOverflow_CAPTURE_LSB_PTR       ((reg16 *) TimerOverflow_TimerHW__CAP0 )
    #define TimerOverflow_PERIOD_LSB          (* (reg16 *) TimerOverflow_TimerHW__PER0 )
    #define TimerOverflow_PERIOD_LSB_PTR        ((reg16 *) TimerOverflow_TimerHW__PER0 )
    #define TimerOverflow_COUNTER_LSB         (* (reg16 *) TimerOverflow_TimerHW__CNT_CMP0 )
    #define TimerOverflow_COUNTER_LSB_PTR       ((reg16 *) TimerOverflow_TimerHW__CNT_CMP0 )


    /***************************************
    *    Register Constants
    ***************************************/

    /* Fixed Function Block Chosen */
    #define TimerOverflow_BLOCK_EN_MASK                     TimerOverflow_TimerHW__PM_ACT_MSK
    #define TimerOverflow_BLOCK_STBY_EN_MASK                TimerOverflow_TimerHW__PM_STBY_MSK

    /* Control Register Bit Locations */
    /* Interrupt Count - Not valid for Fixed Function Block */
    #define TimerOverflow_CTRL_INTCNT_SHIFT                  0x00u
    /* Trigger Polarity - Not valid for Fixed Function Block */
    #define TimerOverflow_CTRL_TRIG_MODE_SHIFT               0x00u
    /* Trigger Enable - Not valid for Fixed Function Block */
    #define TimerOverflow_CTRL_TRIG_EN_SHIFT                 0x00u
    /* Capture Polarity - Not valid for Fixed Function Block */
    #define TimerOverflow_CTRL_CAP_MODE_SHIFT                0x00u
    /* Timer Enable - As defined in Register Map, part of TMRX_CFG0 register */
    #define TimerOverflow_CTRL_ENABLE_SHIFT                  0x00u

    /* Control Register Bit Masks */
    #define TimerOverflow_CTRL_ENABLE                        (0x01u << TimerOverflow_CTRL_ENABLE_SHIFT)

    /* Control2 Register Bit Masks */
    /* As defined in Register Map, Part of the TMRX_CFG1 register */
    #define TimerOverflow_CTRL2_IRQ_SEL_SHIFT                 0x00u
    #define TimerOverflow_CTRL2_IRQ_SEL                      (0x01u << TimerOverflow_CTRL2_IRQ_SEL_SHIFT)

    #if (CY_PSOC5A)
        /* Use CFG1 Mode bits to set run mode */
        /* As defined by Verilog Implementation */
        #define TimerOverflow_CTRL_MODE_SHIFT                     0x01u
        #define TimerOverflow_CTRL_MODE_MASK                     (0x07u << TimerOverflow_CTRL_MODE_SHIFT)
    #endif /* (CY_PSOC5A) */
    #if (CY_PSOC3 || CY_PSOC5LP)
        /* Control3 Register Bit Locations */
        #define TimerOverflow_CTRL_RCOD_SHIFT        0x02u
        #define TimerOverflow_CTRL_ENBL_SHIFT        0x00u
        #define TimerOverflow_CTRL_MODE_SHIFT        0x00u

        /* Control3 Register Bit Masks */
        #define TimerOverflow_CTRL_RCOD_MASK  (0x03u << TimerOverflow_CTRL_RCOD_SHIFT) /* ROD and COD bit masks */
        #define TimerOverflow_CTRL_ENBL_MASK  (0x80u << TimerOverflow_CTRL_ENBL_SHIFT) /* HW_EN bit mask */
        #define TimerOverflow_CTRL_MODE_MASK  (0x03u << TimerOverflow_CTRL_MODE_SHIFT) /* Run mode bit mask */

        #define TimerOverflow_CTRL_RCOD       (0x03u << TimerOverflow_CTRL_RCOD_SHIFT)
        #define TimerOverflow_CTRL_ENBL       (0x80u << TimerOverflow_CTRL_ENBL_SHIFT)
    #endif /* (CY_PSOC3 || CY_PSOC5LP) */

    /*RT1 Synch Constants: Applicable for PSoC3 and PSoC5LP */
    #define TimerOverflow_RT1_SHIFT                       0x04u
    /* Sync TC and CMP bit masks */
    #define TimerOverflow_RT1_MASK                        (0x03u << TimerOverflow_RT1_SHIFT)
    #define TimerOverflow_SYNC                            (0x03u << TimerOverflow_RT1_SHIFT)
    #define TimerOverflow_SYNCDSI_SHIFT                   0x00u
    /* Sync all DSI inputs with Mask  */
    #define TimerOverflow_SYNCDSI_MASK                    (0x0Fu << TimerOverflow_SYNCDSI_SHIFT)
    /* Sync all DSI inputs */
    #define TimerOverflow_SYNCDSI_EN                      (0x0Fu << TimerOverflow_SYNCDSI_SHIFT)

    #define TimerOverflow_CTRL_MODE_PULSEWIDTH            (0x01u << TimerOverflow_CTRL_MODE_SHIFT)
    #define TimerOverflow_CTRL_MODE_PERIOD                (0x02u << TimerOverflow_CTRL_MODE_SHIFT)
    #define TimerOverflow_CTRL_MODE_CONTINUOUS            (0x00u << TimerOverflow_CTRL_MODE_SHIFT)

    /* Status Register Bit Locations */
    /* As defined in Register Map, part of TMRX_SR0 register */
    #define TimerOverflow_STATUS_TC_SHIFT                 0x07u
    /* As defined in Register Map, part of TMRX_SR0 register, Shared with Compare Status */
    #define TimerOverflow_STATUS_CAPTURE_SHIFT            0x06u
    /* As defined in Register Map, part of TMRX_SR0 register */
    #define TimerOverflow_STATUS_TC_INT_MASK_SHIFT        (TimerOverflow_STATUS_TC_SHIFT - 4)
    /* As defined in Register Map, part of TMRX_SR0 register, Shared with Compare Status */
    #define TimerOverflow_STATUS_CAPTURE_INT_MASK_SHIFT   (TimerOverflow_STATUS_CAPTURE_SHIFT - 4)

    /* Status Register Bit Masks */
    #define TimerOverflow_STATUS_TC                       (0x01u << TimerOverflow_STATUS_TC_SHIFT)
    #define TimerOverflow_STATUS_CAPTURE                  (0x01u << TimerOverflow_STATUS_CAPTURE_SHIFT)
    /* Interrupt Enable Bit-Mask for interrupt on TC */
    #define TimerOverflow_STATUS_TC_INT_MASK              (0x01u << TimerOverflow_STATUS_TC_INT_MASK_SHIFT)
    /* Interrupt Enable Bit-Mask for interrupt on Capture */
    #define TimerOverflow_STATUS_CAPTURE_INT_MASK         (0x01u << TimerOverflow_STATUS_CAPTURE_INT_MASK_SHIFT)

#else   /* UDB Registers and Register Constants */


    /***************************************
    *           UDB Registers
    ***************************************/

    #define TimerOverflow_STATUS              (* (reg8 *) TimerOverflow_TimerUDB_rstSts_stsreg__STATUS_REG )
    #define TimerOverflow_STATUS_MASK         (* (reg8 *) TimerOverflow_TimerUDB_rstSts_stsreg__MASK_REG)
    #define TimerOverflow_STATUS_AUX_CTRL     (* (reg8 *) TimerOverflow_TimerUDB_rstSts_stsreg__STATUS_AUX_CTL_REG)
    #define TimerOverflow_CONTROL             (* (reg8 *) TimerOverflow_TimerUDB_sCTRLReg_SyncCtl_ctrlreg__CONTROL_REG )

    #define TimerOverflow_CAPTURE_LSB         (* (reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__F0_REG )
    #define TimerOverflow_CAPTURE_LSB_PTR       ((reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__F0_REG )
    #define TimerOverflow_PERIOD_LSB          (* (reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__D0_REG )
    #define TimerOverflow_PERIOD_LSB_PTR        ((reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__D0_REG )
    #define TimerOverflow_COUNTER_LSB         (* (reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__A0_REG )
    #define TimerOverflow_COUNTER_LSB_PTR       ((reg32 *) TimerOverflow_TimerUDB_sT32_timerdp_u0__A0_REG )

    #if (TimerOverflow_UsingHWCaptureCounter)
        #define TimerOverflow_CAP_COUNT              (*(reg8 *) TimerOverflow_TimerUDB_sCapCount_counter__PERIOD_REG )
        #define TimerOverflow_CAP_COUNT_PTR          ( (reg8 *) TimerOverflow_TimerUDB_sCapCount_counter__PERIOD_REG )
        #define TimerOverflow_CAPTURE_COUNT_CTRL     (*(reg8 *) TimerOverflow_TimerUDB_sCapCount_counter__CONTROL_AUX_CTL_REG )
        #define TimerOverflow_CAPTURE_COUNT_CTRL_PTR ( (reg8 *) TimerOverflow_TimerUDB_sCapCount_counter__CONTROL_AUX_CTL_REG )
    #endif /* (TimerOverflow_UsingHWCaptureCounter) */


    /***************************************
    *       Register Constants
    ***************************************/

    /* Control Register Bit Locations */
    #define TimerOverflow_CTRL_INTCNT_SHIFT              0x00u       /* As defined by Verilog Implementation */
    #define TimerOverflow_CTRL_TRIG_MODE_SHIFT           0x02u       /* As defined by Verilog Implementation */
    #define TimerOverflow_CTRL_TRIG_EN_SHIFT             0x04u       /* As defined by Verilog Implementation */
    #define TimerOverflow_CTRL_CAP_MODE_SHIFT            0x05u       /* As defined by Verilog Implementation */
    #define TimerOverflow_CTRL_ENABLE_SHIFT              0x07u       /* As defined by Verilog Implementation */

    /* Control Register Bit Masks */
    #define TimerOverflow_CTRL_INTCNT_MASK               (0x03u << TimerOverflow_CTRL_INTCNT_SHIFT)
    #define TimerOverflow_CTRL_TRIG_MODE_MASK            (0x03u << TimerOverflow_CTRL_TRIG_MODE_SHIFT)
    #define TimerOverflow_CTRL_TRIG_EN                   (0x01u << TimerOverflow_CTRL_TRIG_EN_SHIFT)
    #define TimerOverflow_CTRL_CAP_MODE_MASK             (0x03u << TimerOverflow_CTRL_CAP_MODE_SHIFT)
    #define TimerOverflow_CTRL_ENABLE                    (0x01u << TimerOverflow_CTRL_ENABLE_SHIFT)

    /* Bit Counter (7-bit) Control Register Bit Definitions */
    /* As defined by the Register map for the AUX Control Register */
    #define TimerOverflow_CNTR_ENABLE                    0x20u

    /* Status Register Bit Locations */
    #define TimerOverflow_STATUS_TC_SHIFT                0x00u  /* As defined by Verilog Implementation */
    #define TimerOverflow_STATUS_CAPTURE_SHIFT           0x01u  /* As defined by Verilog Implementation */
    #define TimerOverflow_STATUS_TC_INT_MASK_SHIFT       TimerOverflow_STATUS_TC_SHIFT
    #define TimerOverflow_STATUS_CAPTURE_INT_MASK_SHIFT  TimerOverflow_STATUS_CAPTURE_SHIFT
    #define TimerOverflow_STATUS_FIFOFULL_SHIFT          0x02u  /* As defined by Verilog Implementation */
    #define TimerOverflow_STATUS_FIFONEMP_SHIFT          0x03u  /* As defined by Verilog Implementation */
    #define TimerOverflow_STATUS_FIFOFULL_INT_MASK_SHIFT TimerOverflow_STATUS_FIFOFULL_SHIFT

    /* Status Register Bit Masks */
    /* Sticky TC Event Bit-Mask */
    #define TimerOverflow_STATUS_TC                      (0x01u << TimerOverflow_STATUS_TC_SHIFT)
    /* Sticky Capture Event Bit-Mask */
    #define TimerOverflow_STATUS_CAPTURE                 (0x01u << TimerOverflow_STATUS_CAPTURE_SHIFT)
    /* Interrupt Enable Bit-Mask */
    #define TimerOverflow_STATUS_TC_INT_MASK             (0x01u << TimerOverflow_STATUS_TC_SHIFT)
    /* Interrupt Enable Bit-Mask */
    #define TimerOverflow_STATUS_CAPTURE_INT_MASK        (0x01u << TimerOverflow_STATUS_CAPTURE_SHIFT)
    /* NOT-Sticky FIFO Full Bit-Mask */
    #define TimerOverflow_STATUS_FIFOFULL                (0x01u << TimerOverflow_STATUS_FIFOFULL_SHIFT)
    /* NOT-Sticky FIFO Not Empty Bit-Mask */
    #define TimerOverflow_STATUS_FIFONEMP                (0x01u << TimerOverflow_STATUS_FIFONEMP_SHIFT)
    /* Interrupt Enable Bit-Mask */
    #define TimerOverflow_STATUS_FIFOFULL_INT_MASK       (0x01u << TimerOverflow_STATUS_FIFOFULL_SHIFT)

    #define TimerOverflow_STATUS_ACTL_INT_EN             0x10u   /* As defined for the ACTL Register */

    /* Datapath Auxillary Control Register definitions */
    #define TimerOverflow_AUX_CTRL_FIFO0_CLR             0x01u   /* As defined by Register map */
    #define TimerOverflow_AUX_CTRL_FIFO1_CLR             0x02u   /* As defined by Register map */
    #define TimerOverflow_AUX_CTRL_FIFO0_LVL             0x04u   /* As defined by Register map */
    #define TimerOverflow_AUX_CTRL_FIFO1_LVL             0x08u   /* As defined by Register map */
    #define TimerOverflow_STATUS_ACTL_INT_EN_MASK        0x10u   /* As defined for the ACTL Register */

#endif /* Implementation Specific Registers and Register Constants */

#endif  /* CY_Timer_v2_30_TimerOverflow_H */


/* [] END OF FILE */
